@@ -2,6 +2,8 @@
 
 module TestAdjointTranspose
 
+isdefined(Main, :pruned_old_LA) || @eval Main include("prune_old_LA.jl")
+
 using Test, LinearAlgebra
 
 const BASE_TEST_PATH = joinpath(Sys.BINDIR, "..", "share", "julia", "test")
@@ -308,6 +310,35 @@ end
     @test vec(adjoint(mvec))[1] == adjoint(mvec[1])
 end
 
+@testset "Adjoint and Transpose view methods" begin
+    intvec, intmat = [1, 2], [1 2 3; 4 5 6]
+    # overload of reshape(v, Val(1)) simplifies views of row vectors:
+    @test view(adjoint(intvec), 1:2) isa SubArray{Int, 1, Vector{Int}}
+    @test view(transpose(intvec), 1:2) isa SubArray{Int, 1, Vector{Int}}
+    cvec = [1, 2im, 3, 4im]
+    @test view(transpose(cvec), 2:3) === view(cvec, 2:3)
+    @test view(adjoint(cvec), 2:3) == conj(view(cvec, 2:3))
+
+    # vector slices of transposed matrices are simplified:
+    @test view(adjoint(intmat), 1, :) isa SubArray{Int, 1, Matrix{Int}}
+    @test view(transpose(intmat), 1, :) isa SubArray{Int, 1, Matrix{Int}}
+    @test view(adjoint(intmat), 1, :) == permutedims(intmat)[1, :]
+    @test view(transpose(intmat), 1:1, :) == permutedims(intmat)[1:1, :] # not simplified
+    @test view(adjoint(intmat), :, 2) isa SubArray{Int, 1, Matrix{Int}}
+    @test view(transpose(intmat), :, 2) isa SubArray{Int, 1, Matrix{Int}}
+    @test view(adjoint(intmat), :, 2) == permutedims(intmat)[:, 2]
+    @test view(transpose(intmat), :, 2:2) == permutedims(intmat)[:, 2:2] # not simplified
+    cmat = [1 2im 3; 4im 5 6im]
+    @test view(transpose(cmat), 1, :) isa SubArray{Complex{Int}, 1, Matrix{Complex{Int}}}
+    @test view(transpose(cmat), :, 2) == cmat[2, :]
+    @test view(adjoint(cmat), :, 2) == conj(cmat[2, :]) # not simplified
+
+    # bounds checks happen before this
+    @test_throws BoundsError view(adjoint(intvec), 0:3)
+    @test_throws BoundsError view(transpose(cvec), 0:3)
+    @test_throws BoundsError view(adjoint(intmat), :, 3)
+end
+
 @testset "horizontal concatenation of Adjoint/Transpose-wrapped vectors and Numbers" begin
     # horizontal concatenation of Adjoint/Transpose-wrapped vectors and Numbers
     # should preserve the Adjoint/Transpose-wrapper to preserve semantics downstream
@@ -532,6 +563,11 @@ end
     @test String(take!(io)) == "transpose(::Matrix{Float64})"
 end
 
+@testset "show" begin
+    @test repr(adjoint([1,2,3])) == "adjoint([1, 2, 3])"
+    @test repr(transpose([1f0,2f0])) == "transpose(Float32[1.0, 2.0])"
+end
+
 @testset "strided transposes" begin
     for t in (Adjoint, Transpose)
         @test strides(t(rand(3))) == (3, 1)
@@ -713,4 +749,26 @@ end
     end
 end
 
+@testset "diagview" begin
+    for A in (rand(4, 4), rand(ComplexF64,4,4),
+                fill([1 2; 3 4], 4, 4))
+        for k in -3:3
+            @test diagview(A', k) == diag(A', k)
+            @test diagview(transpose(A), k) == diag(transpose(A), k)
+        end
+        @test IndexStyle(diagview(A')) == IndexLinear()
+    end
+end
+
+@testset "triu!/tril!" begin
+    @testset for sz in ((4,4), (3,4), (4,3))
+        A = rand(sz...)
+        B = similar(A)
+        @testset for f in (adjoint, transpose), k in -3:3
+            @test triu!(f(copy!(B, A)), k) == triu(f(A), k)
+            @test tril!(f(copy!(B, A)), k) == tril!(f(A), k)
+        end
+    end
+end
+
 end # module TestAdjointTranspose
